../../pluto/bin/ipsec-look.sh
# ==== cut ====
ipsec auto --status | grep westnet-eastnet-compress
# ==== tuc ====
../bin/check-for-core.sh
if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
