/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note
 * this is from linux/if_link.h commit f203b76d7
 */

/* XFRM section */
enum {
        IFLA_XFRM_UNSPEC,
        IFLA_XFRM_LINK,
        IFLA_XFRM_IF_ID,
        __IFLA_XFRM_MAX
};
